#[derive(Debug, Clone)]
pub enum Instruction {
    A{val: i16},
    C{dest:String, comp:String, jump:String},
}

pub fn parse(lines : Vec<(usize, String)>) -> Vec<(usize,Instruction)> {
    let mut instructions: Vec<(usize, Instruction)> = Vec::new();

    for (i, line) in lines {
        match line_to_instruction(line) {
            None => {
                println!("ERROR on line {}", i+1);
                println!("Invalid Syntax");
                std::process::exit(-1);
            },
            Some(instruction) => instructions.push((i, instruction))
        };
    }

    instructions
}

fn line_to_instruction(line: String) -> Option<Instruction> {
    if &line[0..1] == "@" {
        //A instruction
        let value :i16 = match line[1..].trim().parse() {
            Err(_) => return None,
            Ok(val) => val,
        };

        Some(Instruction::A{val: value})
    }
    else {
        //C instruction

        // dest
        let equals = line.find("=");

        let dest = match equals {
            None => String::from(""),
            Some(i) => String::from(&line[..i])
        };

        // comp
        let semicolon = line.find(";");

        let comp = match equals {
            None => {
                match semicolon {
                    None => return None,
                    Some(i) => String::from(&line[..i]),
                }
            }
            Some (i) => {
                match semicolon {
                    None => String::from(&line[i+1..]),
                    Some(j) => String::from(&line[i+1..j]),
                }
            }
        };

        // jump
        let jump = match semicolon {
            None => String::from(""),
            Some(i) => String::from(&line[i+1..])
        };

        Some(Instruction::C{dest, comp, jump})
    }
}