use hack16::{ preprocessor, parser, parser::Instruction};

mod simulator;

fn main() {
    let args: Vec<String> = std::env::args().collect();

    if args.len() < 5 {
        usage();
    }

    let path = String::from(&args[1]);
    let lines = preprocessor::preprocess(path);
    let instructions: Vec<(usize,Instruction)> = parser::parse(lines);
    
    let mut debug = false;

    let clock = match String::from(&args[2]).parse::<u64>() {
        Ok(clock) => clock,
        _ => usage(),
    };

    let tolerance = match String::from(&args[3]).parse::<f64>() {
        Ok(tolerance) => tolerance,
        _ => usage(),
    };

    let fps = match String::from(&args[4]).parse::<u64>() {
        Ok(fps) => fps,
        _ => usage(),
    };

    if args.len() > 5 {
        for arg in &args[5..args.len()] {
            match arg.as_str() {
                "-d" => debug = true,
                flag => {
                    println!("ERROR: unrecognized flag: {} ", flag);
                    usage();
                },
            }
        }
    }

    let mut machine = simulator::Machine::new(clock, tolerance, fps);
    machine.load_rom(instructions);

    machine.set_debug(debug);

    machine.set_ram(0, 4);
    machine.set_ram(1, 5);

    let final_state = machine.start();

    println!("{:?}", &final_state[0..15]);
}

fn usage() -> ! {
    println!("USAGE: sim [path/to/prog.asm] [clock rate] [tolerance] [update rate] [options]");
    println!("Clock rate is number of cycles per second in Hz - warnings will be generated if your machine is unable to maintain this speed.");
    println!("Tolerance specifies how much longer then 1 second per clock rate cycles is acceptable");
    println!("Update rate specifies how many times the screen should be updated per second");
    println!("Options:");
    println!("-d\tDebug mode");
    std::process::exit(-1);
}